#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import time
import argparse

import technoap.device.apg7300 as usbmca


##############################################################
def main(args):
    """ main routine

        args: arguments
    """

    print("start!!!")

    try:
        # open device
        serialnum = ("APG7300AA").encode("utf-8")
        handle = usbmca.apg7300.APG7300.OpenDeviceBySerialNumber(serialnum)

        if handle is not None:
            # initialize device
            handle.InitialDevice()

            # configure
            config = handle.createConfigObject()
            result = set_config(config)

            if result == True:
                acquisition(handle, config, args.time, args.file)
            else:
                print("configuration error...")
        else:
            print("device open error(%s)..." % (serialnum))
    except Exception as ex:
        print(ex)

    print("\nfinish!!!")


##############################################################
def acquisition(handle, config, acq_sec, savefile):
    """ measurement processing

        handle: device handle
        config: config object
        acq_sec: measurement time(sec)
        savefile: name of the file to save the histogram data
    """

    # issue a stop command once considering that the measurement is continuing
    config.stopAcquisition()

    config.clearAcquisitionData()   # clear the data held on the device side

    config.setAcquisitionMode(0)        # 0=histogram mode(fixed)
    config.setAcquisitionTimeMode(0)    # 0=real time mode, 1=live time mode
    config.setAcquisitionTime(acq_sec)  # set acquisition time(seconds)

    elapsedTime = -0.001
    status = handle.createStatusObject()

    # start acquisition
    config.startAcquisition()

    ########## acquisition ##########
    while acq_sec >= elapsedTime:
        status.readStatus()
        elapsedTime = status.getRealTimeSec()

        print("\n##### %.1f/%.1f(sec) #####" % (elapsedTime, acq_sec))
        print("live time(sec):    %f" % status.getLiveTimeSec(usbmca.constant.CHNUM_1ST))
        print("dead time(sec):    %f" % status.getDeadTimeSec(usbmca.constant.CHNUM_1ST))
        print("throughput total count: %d" % status.mStatus["TCT1",])
        print("throughput rate (cps):  %d" % status.mStatus["TCR1",])

        time.sleep(0.5)  # wait

    # issue a acquisition stop command upon completion
    dummy = config.stopAcquisition()

    ########## read histogram and write as csv file ##########
    histdata = []
    histogram = handle.createHistogramObject()

    result, hist0 = histogram.read()
    if result <= 0:
        print("\nFailed to read histogram data(return code = %d)" % result)
        return

    histdata.append(hist0)

    with open(savefile, "w") as csv:
        csv.writelines("[header]\n")
        csv.writelines("real time(s),%f\n" % (elapsedTime))

        csv.writelines("[data]\n")
        csv.writelines("#bin,CH1\n")

        for i in range(16384):
            textline = "%d,%d\n" % (i, histdata[0][i])
            csv.writelines(textline)

    return


##############################################################
def set_config(config):
    """ configure device

        config: config object
    """
    ### prepare parameter
    config.mParameter["PDSW",] = 1      # peak detect: 0=absolute, 1=fast
    config.mParameter["ADG",] = [0]     # ADC gain: 0=16384bin, 1=8192bin, ..., 5=512bin
    config.mParameter["THR",] = [40]    # threshold
    config.mParameter["LLD",] = [40]
    config.mParameter["ULD",] = [16383]
    config.mParameter["OFS",] = [0]     # offset

    ### write parameter
    result = config.writeParameterAll()

    return result


##############################################################
if __name__ == "__main__":
    msg_file1 = "specify the measurement time(sec). Default: 5 sec."
    msg_file2 = 'specify the file (CSV format) to save the histogram data. Default: "./usbmca.csv".'
    parser = argparse.ArgumentParser(description="Acquisition in histogram mode.")
    parser.add_argument("-t", "--time", default="5", type=int, help=msg_file1)
    parser.add_argument("-f", "--file", default="./usbmca.csv", help=msg_file2)
    args = parser.parse_args()

    result = usbmca.apg7300.startup()

    if result == True:
        main(args)
        result = usbmca.apg7300.cleanup()
